package xpug.kata.birthday_greetings;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;

import javax.mail.MessagingException;
import javax.mail.internet.AddressException;

public class BirthdayService {

	private final EmployeeFacade employeeFacade;
	private final GreetingsFacade greetingsFacade;

	public BirthdayService(EmployeeFacade employeeFacade, GreetingsFacade greetingsFacade) {
		this.employeeFacade = employeeFacade;
		this.greetingsFacade = greetingsFacade;
	}

	public void sendGreetings(OurDate ourDate) throws IOException, ParseException, AddressException, MessagingException {

		List<Employee> employees = employeeFacade.loadEmployees();

		for (Employee employee : On.items(employees).collect(Being.bornOn(ourDate))) {
			sendGreetingTo(employee, happyBirthday(employee));
		}
		
		for (Employee employee : On.items(employees).collect(Being.female())) {
			sendGreetingTo(employee, "A kiss for you");
		}
	}

	private void sendGreetingTo(Employee employee, String body) throws AddressException, MessagingException {
		String receiver = employee.getEmail();
		String subject = "Happy Birthday!";

		Greeting greeting = new Greeting(subject, body, receiver);
		greetingsFacade.send(greeting);
	}

	private String happyBirthday(Employee employee) {
		String body = "Happy Birthday, dear %NAME%!".replace("%NAME%", employee.getFirstName());
		return body;
	}

	public static void main(String[] args) {
		EmployeeFacade employeeFacade = new FileSystemEmployeeFacadeAdapter("employee_data.txt");
		GreetingsFacade greetingsFacade = new SMTPGreetingsFacadeAdapter("localhost", 25, "sender@here.com");
		
		BirthdayService service = new BirthdayService(employeeFacade,greetingsFacade);

		try {
			service.sendGreetings(new OurDate("2008/10/08"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
