package xpug.kata.birthday_greetings;

import static org.junit.Assert.assertEquals;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.mail.MessagingException;
import javax.mail.internet.AddressException;

import org.junit.Test;

public class BirthdayServiceTest {

	private final class MockGreetingsFacade implements GreetingsFacade {
		private final List<Greeting> sentGreetings;

		private MockGreetingsFacade(List<Greeting> sentGreetings) {
			this.sentGreetings = sentGreetings;
		}

		public void send(Greeting greeting) throws AddressException, MessagingException {
			sentGreetings.add(greeting); 
		}
	}

	private final class MockEmployeeFacade implements EmployeeFacade {
		private final List<Employee> eployees;

		private MockEmployeeFacade(List<Employee> eployees) {
			this.eployees = eployees;
		}

		public List<Employee> loadEmployees() throws FileNotFoundException, IOException, ParseException {
			return eployees;
		}
	}

	@Test
	public void testSendGreetings() throws Exception {
		final String miloesBirthday = "10/12/2009";
		
		final List<Employee> eployees = Arrays.asList(
				new Employee("Milo", "", miloesBirthday, "", Gender.Male),
				new Employee("Helen", "", "15/12/1945", "", Gender.Female)
		);
		final List<Greeting> sentGreetings = new ArrayList<Greeting>();
		
		BirthdayService birthdayService = new BirthdayService(new MockEmployeeFacade(eployees), new MockGreetingsFacade(sentGreetings));
		birthdayService.sendGreetings(new OurDate(miloesBirthday));
		
		
		assertEquals(2, sentGreetings.size());
		assertEquals("Happy Birthday, dear Milo!", sentGreetings.get(0).getBody());
		assertEquals("A kiss for you", sentGreetings.get(1).getBody());
	}

}
