package xpug.kata.birthday_greetings;

import java.util.ArrayList;
import java.util.List;

public class InternalIterator<TYPE> {

	private final List<TYPE> items;

	public InternalIterator(List<TYPE> items) {
		this.items = items;
	}

	public List<TYPE> collect(Predicate<TYPE> predicate) {
		List<TYPE> result = new ArrayList<TYPE>();
		for (TYPE eachItem : items) {
			if (predicate.evaluate(eachItem)) {
				result.add(eachItem);
			}
		}
		return result;
	}

	public TYPE findFirst(Predicate<TYPE> predicate) {
		List<TYPE> result = collect(predicate);
		return result.isEmpty() ? null : result.get(0);
	}

	public List<TYPE> reject(Predicate<TYPE> predicate) {
		return collect(new Not<TYPE>(predicate));
	}

	public boolean contains(Predicate<TYPE> predicate) {
		return ! collect(predicate).isEmpty();
	}

	public <TO> List<TO> apply(Transformation<TYPE, TO> tranfromation) {
		List<TO> result = new ArrayList<TO>();
		for (TYPE eachItem : items) {
			result.add(tranfromation.applyOn(eachItem));
		}
		return result;
	}
}
